function runFDConvergence(optObjFunc, parSetFunc, invPars, parSetOpts, deltaParExps, order, dim)

default_arg('dim', 2);
default_arg('order', 2);
default_arg('deltaParExps', 0:-1/2:-12);
default_arg('parSetOpts', struct);
default_arg('invPars', {'rho'});
default_arg('parSetFunc', @pars.rhoInversionParsReceiverLine);
default_arg('optObjFunc', @Elastic2DOpt);

[parset, trueParset] = parSetFunc(parSetOpts);

% Run with true parameters and save data
trueOptObj = optObjFunc(trueParset, invPars, order);
trueOptObj.runForward(false);
trueReceiverData = trueOptObj.forwardReceiverRecordings;
k = trueOptObj.k;
clear trueOptObj;

% Create optimization object
adjOpt = optObjFunc(parset, invPars, order, k);
adjOpt.receiverData = trueReceiverData;

% Compute gradient with adjoint method
grad = adjOpt.computeGradient();

% Compute gradient with brute force FD
DG = 10.^deltaParExps;
err = zeros(size(DG));

tic
str = util.replace_string('','   %d %%',0);
for i = 1:length(DG)
	dg = DG(i);
	gradFD = adjOpt.computeGradientFD(dg);

	% Add errors from different parts of gradient
	err(i) = 0;
	for j = 1:numel(invPars)
		switch invPars{j}
		case {'rho', 'lambda', 'mu'}
			err(i) = err(i) + norm(gradFD{j}-grad{j}) / norm(grad{j});
		case 'source'
			% Compute sum of l2-errors
			nSources = numel(parset.sources.x);
			for s = 1:nSources
				for d = 1:dim
					err(i) = err(i) + norm(gradFD{j}{s,d}-grad{j}{s,d})/norm(grad{j}{s,d});
				end
			end
		end
	end
	str = util.replace_string(str,'   %.2f %%',i/length(DG)*100);
end
str = util.replace_string(str,'');
toc

figure
loglog(DG, err, '-');
hold on
loglog(DG, DG, '-k')
hold off
xlabel('\Delta parameter')
ylabel('Relative l^2-error')
legend('FD gradient vs adjoint','1st order ref')