function [pars, truePars] = rhoInversionPars(opts)

% Default values
default_arg('opts',struct);
opts = defaultField(opts, 'pertSize', 100);
opts = defaultField(opts, 'xlen', 51);
opts = defaultField(opts, 'xs', 250);
opts = defaultField(opts, 'xr', 800);
opts = defaultField(opts, 'T', 2.0);
opts = defaultField(opts, 'lim', {0,1000});

pertSize = opts.pertSize;
xlen     = opts.xlen;
lim      = opts.lim;
xs       = opts.xs;
xr       = opts.xr;
T        = opts.T;

% Create source time function
t0 = 0.5;
a = 1.0; c = 0.1; sc = 1/(2*c^2);
src = @(t) a*exp(-(t-t0).*(t-t0)*sc);
source = struct;
source.dat.p = src;
source.dat.v = [];
source.x = xs; 

% Create receiver
receiver = struct;
receiver.dat = [];
receiver.x = xr;

[g_primal, g_dual] = grid.primalDual1D(xlen, lim);
x_primal = g_primal.points();
x_dual = g_dual.points();

% Material properties
rholeft  = 1000; rhoright = 2000;
cleft = 1500; cright = 3000;
Kleft = rholeft*cleft^2; Kright = rhoright*cright^2;

% True material properties
rhofunc = @(x)rholeft + (rhoright-rholeft)*x/(lim{2} - lim{1});
rho = rhofunc(x_dual);
Kfunc = @(x)Kleft + (Kright-Kleft)*x/(lim{2} - lim{1});
K = Kfunc(x_primal);

% Perturbed properties
rhofuncptb = @(x)(rholeft + (rhoright-rholeft)*x/(lim{2} - lim{1})) - pertSize;
rhoptb = rhofuncptb(x_dual);

% Return struct
pars.xlen = xlen;
pars.lim = lim;
pars.fsource = source;
pars.receiver = receiver;
pars.T   = T;
pars.rho = rho;
pars.K   = K;

truePars = pars;
pars.rho = rhoptb;
